@@ -3,7 +3,8 @@ from .affine import *
 from .atbash import *
 from .bacon import *
 from .barbie import *
-from .rotn import *
+from .rot import *
+from .scytale import *
 from .shift import *
-from .xor_byte import *
+from .xor import *
 
